C ESA (C) 2000-2018 
C
C This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
C
C    SAS is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.
C
C    SAS is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.
C
C    You should have received a copy of the GNU General Public License
C    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
      SUBROUTINE DATREV (NFILE,DATE,NREV,TRS,TRE,IERR)

C     INTERFACE ROUTINE FOR THE REVNO FILE, I.E.
C               DATREV WITH DATE AS INPUT AND TIME INTERVAL FOR CORRESP.
C                      REVOLUTION AS OUTPUT,
C               REVDAT WITH REVOLUTION NO. AS INPUT AND TIME INTERVAL OF
C                      THAT REVOLUTION AS OUTPUT.
C
C     THE CALLING SEQUENCE IS:
C               NFILE (INPUT,  I*4)    UNIT NUMBER (NONNEGATIVE)
C               DATE  (INPUT,  R*8)    DATE IN MJD2000
C               NREV  (OUTPUT, I*4)    REVOLUTION NUMBER AND CORRESPONDING
C               TRS   (OUTPUT, R*8)    TIME INTERVAL IN MJD2000,
C               TRE   (OUTPUT, R*8)    TRS = START AND TRE = END
C               IERR  (OUTPUT, I*4)    ERROR NUMBER, SEE ICD
C
      IMPLICIT REAL*8 (A-H,O-Z)
      CHARACTER*1     RP
      CHARACTER*60    FMT
      DIMENSION       NTA(6),NTB(6),TIMREV(100),NUMREV(100)
      DATA NFILEO /0/
      
      IF(NFILE.EQ.NFILEO) GOTO 1
      IROUT=+1
      GOTO 30
    1 IF(DATE.LT.TFS) GOTO 101
      IF(DATE.GE.TFE) GOTO 102
      IF(DATE.GE.TIMREV(   1)) GOTO 2
      IROUT=+1
      GOTO 30
    2 IF(DATE.LT.TIMREV(MREC)) GOTO 3
      IROUT=+1
      GOTO 40
      
    3 PERIOD=(TIMREV(MREC)-TIMREV(1))/(MREC-1)
      NREC=(DATE-TIMREV(1))/PERIOD + 1.D0
      NREC=MIN(MREC,MAX(1,NREC))
      IF(DATE.GE.TIMREV(NREC)) GOTO 5
      DO 4 I=NREC,1,-1
      IF(DATE.LT.TIMREV(I)) GOTO 4
      NREV=NUMREV(I)
      TRS=TIMREV(I  )
      TRE=TIMREV(I+1)
      GOTO 20
    4 CONTINUE
      GOTO 105
    5 DO 6 I=NREC,MREC,+1
      IF(DATE.GE.TIMREV(I)) GOTO 6
      NREV=NUMREV(I-1)
      TRS=TIMREV(I-1)
      TRE=TIMREV(I  )
      GOTO 20
    6 CONTINUE
      GOTO 105

      ENTRY      REVDAT (NFILE,     NREV,TRS,TRE,IERR)
      
C     THE CALLING SEQUENCE IS AS ABOVE WITH THE EXCEPTION OF:
C               NREV  (INPUT,  I*4)    REVOLUTION NUMBER
      
      IF(NFILE.EQ.NFILEO) GOTO 11
      IROUT=-1
      GOTO 30
   11 IF(NREV.LT.   1) GOTO 101
      IF(NREV.GE.LREV) GOTO 102
      IF(NREV.GE.NUMREV(   1)) GOTO 12
      IROUT=-1
      GOTO 30
   12 IF(NREV.LT.NUMREV(MREC)) GOTO 13 
      IROUT=-1
      GOTO 40
      
   13 NREC=NREV-NUMREV(1)+1
      TRS=TIMREV(NREC)
      TRE=TIMREV(NREC+1)
   20 IERR=0
      RETURN
      
   30 IF(NFILE.LT.0) GOTO 106
      REWIND NFILE
      IEND=0
      READ(NFILE,FMT='(/)',ERR=106,END=106)
      READ(NFILE,FMT='(2(I6,1X,I2,1X,I2,I4,1X,I2,1X,I2),2I6)',ERR=106,
     *     END=106) NTA,NTB,LREC,NEOF
      LREV=LREC-6
      CALL JD2000 (TFS,NTA(1),NTA(2),NTA(3),NTA(4),NTA(5),DBLE(NTA(6)))
      CALL JD2000 (TFE,NTB(1),NTB(2),NTB(3),NTB(4),NTB(5),DBLE(NTB(6)))
      READ(NFILE,FMT='(A60)',ERR=106,END=106) FMT
      READ(NFILE,FMT='(/)',ERR=106,END=106)
      NFILEO=NFILE
      READ(NFILE,FMT=FMT,ERR=106,END=106) INY,NUMSC,RP,NY,NM,ND,NH,NMI,
     *                                    NSEC,NUMREV(1)
      IF(NUMREV(1).NE.1) GOTO 105
      CALL JD2000 (TIMREV(1),NY,NM,ND,NH,NMI,DBLE(NSEC))
      N=2
   31 IF(IEND.EQ.1) GOTO 102
      DO 32 I=N,100
      READ(NFILE,FMT=FMT,ERR=106,END= 33) INY,NUMSC,RP,NY,NM,ND,NH,NMI,
     *                                    NSEC,NUMREV(I)
      IF(NUMREV(I).EQ.0) GOTO 33
      IF(NUMREV(I).NE.NUMREV(I-1)+1) GOTO 105
      CALL JD2000 (TIMREV(I),NY,NM,ND,NH,NMI,DBLE(NSEC))
   32 MREC=I
      GOTO 34
   33 IEND=1
      MREC=I-1
   34 IF (IROUT) 11,105,1
      
   40 DO 41 I=1,2
      TIMREV(I)=TIMREV(MREC-2+I)
   41 NUMREV(I)=NUMREV(MREC-2+I)
      N=3
      GOTO 31
      
  101 IERR=1
      GOTO 110
  102 IERR=2
      GOTO 110
  105 IERR=5
      GOTO 110
  106 IERR=6
  110 NFILEO=0
      RETURN
      END
