// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

#ifndef FDSORBIT_H
#define FDSORBIT_H

extern "C" void staton_(int    &mfile,  // I: LUN of Station coord file
			int    &nsta,   // I: Station Number
			double *r,      // O: double[4]: VECTOR CONTAINING GEOCENTRIC 
                                        //               EARTH-ROTATING CARTESIAN COORDINATES 
			                //               AND GEOCENTRIC RADIUS VECTOR 
			double *an,     // O: double[7]: VECTOR CONTAINING STATION ALTITUDE 
			                //               ABOVE GEOID, LATITUDE, ITS SINE AND COSINE, 
			                //               LONGITUDE(EASTERN), ITS SINE AND COSINE 
			double *tda,    // O: double[9]: MATRIX FOR TRANSFORMING GEOCENTRIC EARTH-
			                //               ROTATING COORDINATES(X) BY SUM TDA(J,K)*X(K), 
			                //               K=1,3  TO TOPOCENTRIC COORDINATES: EAST, NORTH, ZENITH
			char   *aname,  // O: char[8]: ALPHANUMERIC STATION NAME       
			int    *ierr);  // O: int: RETURN CODE, 
                                        //         0    = NO ERROR.   
                                        //         2801 = NOT EXISTING STATION NR.  
                                        //         2802 = READ ERROR FROM STACO FILE.  

extern "C" void sigdel_(int    &lfile,  // I: LUN of Orbit file
			int    &mfile,  // I: LUN of Station file
			int    &nsta,   // I: Station number
			double &day,    // I: MJD2000 OF EPOCH
			double *delay,  // O: double: SIGNAL TIME (SEC) FROM SATELLITE TO GROUND STATION
			int    *nsat,   // O: int: SATELLITE NUMBER
			int    *ierr);  // O: int: RETURN CODE FROM ORBIT FILE: 
                                        //         0 = NO ERROR, 
                                        //         1 = DAY TOO EARLY,
                                        //         2 = DAY TOO LATE, 
                                        //         3 = TIME GAP IN DATA,
                                        //         4 = NOT EXISTING STATION NR. IN STATION COORD. FILE 
                                        //         5 = ORBIT FILE CONTENT INCONSISTENT, 
                                        //         6 = READ ERROR FROM DATA FILE

extern "C" void orbita_(double &day,    // I: MODIFIED JULIAN DAY, FROM 2000, FOR THE STATE VECTOR
			int    &kode,   // I: NUMBER OF COMPONENTS OF STATE VECTOR = DIM. OF ARRAY X(); 
                                        //    = 3 FOR S/C POSITION, = 6 FOR POSITION & VELOCITY
			int    &lfile,  // I: LOGICAL NUMBER OF INPUT DATA FILE
			int    *ierr,   // O: int: RETURN CODE: 
			                //         0 = NO ERROR, 
			                //         1 = DAY TOO EARLY, 
			                //         2 = TOO LATE,  =TIME GAP IN DATA, 
			                //         3 = WRONG VALUE OF 'KODE',
			                //         4 = FILE CONTENT INCONSISTENT, 
			                //         5 = READ ERROR FROM DATA FILE ,
			                //         7 = WRONG UNIT NUMBER
			int    *nsta,   // O: int: SATELLITE NUMBER: FROM 1 to 99
			double *x,      // O: double[3|6]: (KODE) SPACECRAFT POSITION, KM (AND VELOCITY, KM/S)
			double *rev);   // O: double: REVOLUTION NUMBER


extern "C" void gj2efs_(double &day,    // I: EPOCH IN MODIFIED JULIAN DATE FROM 2000
			double *xj2000, // I: double[3]: S/C POSITION IN THE J-2000 SYSTEM (KM)
			double *xefs);  // O: double[3]: S/C POSITION IN THE PSEUDO-EARTH FIXED SYSTEM

extern "C" void geolat_(double *x,      // I: double[3]: S/C POSITION IN THE PSEUDO-EARTH FIXED SYSTEM
			double *xlong,  // O: double: LONGITUDE (RAD)
			double *xlat,   // O: double: LATITUDE (RAD)
			double *height);// O: double: HEIGHT ABOVE EARTH (KM)
/*
extern "C" void datrev_(int    &nfile,  // I: int: UNIT NUMBER (NONNEGATIVE)
			double &date,   // I: MODIFIED JULIAN DAY, FROM 2000, FOR THE STATE VECTOR
			int    *rev,    // O: int Revolution number
			double *trs,    // O: double: Rev start
			double *tre,    // O: double: Rev end
			int    *ierr);  // O: int: Error code

extern "C" void revdat_(int    &nfile,  // I: int: UNIT NUMBER (NONNEGATIVE)
			int    &rev,    // O: int Revolution number
			double *trs,    // O: double: Rev start
			double *tre,    // O: double: Rev end
			int    *ierr);  // O: int: Error code
*/
#endif

