C ESA (C) 2000-2018 
C
C This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
C
C    SAS is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.
C
C    SAS is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.
C
C    You should have received a copy of the GNU General Public License
C    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
      SUBROUTINE ORBITA(DAY,KODE,LFILE,IERROR,NSAT,X,REVNUM)
CP  ORBIT: RETRIEVAL ROUTINE FOR COMPRESSED LTOF/STOF ORBIT FILE 
C   UPDATE OF 1994/08/30
C   UPDATE OF 1995/03/24: REMOVE REFERENCE TO CLUSTER
C                         CHANGE SEARCH FOR FIRST BLOCK RECORD
C                         RETURN EARLIEST/LATEST DATES IF INVALID DAY
C 
C   INPUT:
CI  DAY (R*8) = MODIFIED JULIAN DAY, FROM 2000, FOR THE STATE VECTOR
CI  KODE (I*4) = NUMBER OF COMPONENTS OF STATE VECTOR = DIM. OF ARRAY
C              X(); = 3 FOR S/C POSITION, = 6 FOR POSITION & VELOCITY
CI  LFILE (I*4) = LOGICAL NUMBER OF INPUT DATA FILE
C   OUTPUT:
CO  IERROR (I*4) = RETURN CODE: 0=NO ERROR, 1='DAY' TOO EARLY, 2=TOO
C            LATE,  =TIME GAP IN DATA, 3=WRONG VALUE OF 'KODE',
C            4=FILE CONTENT INCONSISTENT, 5=READ ERROR FROM DATA FILE ,
C            7=WRONG UNIT NUMBER
CO  NSAT (I*4) = SATELLITE NUMBER: FROM 1 to 99
CO  X(KODE) (R*8) = SPACECRAFT POSITION, KM (AND VELOCITY, KM/S)
CO                  IF 'IERROR' = 1 THEN X(2) = EARLIEST TIME (MJD2000)
CO                  IF 'IERROR' = 2 THEN X(3) = LATEST TIME (MJD2000)
CO  REVNUM (R*8) = REVOLUTION NUMBER
C
CF  READS A SEQUENTIAL FORMATTED FILE WITH LOGICAL NUMBER 'LFILE'
      IMPLICIT REAL*8(A-H,O-Z)
C                                                                               
C
      DIMENSION Y(6),COEFF(10,6),X(KODE)
      CHARACTER*3 CH3
C   INITIALISE FILE NUMBER TO FORCE FILE READING AT FIRST CALL
      DATA MFILE/-9999/
C
C  PARAMETERS SAVED INSIDE ROUTINE: NSATIN,MFILE,DAYFIR,DAYLAS,Y(),
C  RDIST,COEFF(,),KOEFF,DAYBEG,DAYEND,EPOCH,REVEPO,SMAXIS,OMOTIN
      SAVE
C
C  INITIALISE ERROR CODES
      IERROR = 0
      IF(KODE .LE. 0) GOTO 503
      IF(KODE .GT. 6) GOTO 503
      IF(LFILE.LT. 1) GOTO 507
C
C  ALWAYS REWIND IF A NEW FILE NUMBER IS USED
      IF(LFILE .NE. MFILE) GOTO 10
C
C  CHECK IF 'DAY' IS INSIDE LAST READ RECORD BLOCK
      IF(DAY .GT. DAYEND + 1.D-4) GOTO 20
      IF(DAY .GE. DAYBEG - 1.D-4) GOTO 70
C
C  INITIALISE THE READING FROM THE FILE
10    DAYFIR = 99.D9
      DAYLAS = 99.D9
      MFILE = LFILE
      REWIND LFILE
C
20    CONTINUE
C  READ 1ST RECORD IN A BLOCK
      READ(LFILE,41,ERR=505,END=509) NSATIN, CH3
41    FORMAT(I3,A3)
C  IF: CH3 IS '  P' OR '  R' THEN THIS IS 1ST RECORD IN A BLOCK
      IF(CH3.NE.'  P' .AND. CH3.NE.'  R') GOTO 20
C
C  READ 2ND RECORD IN THE BLOCK
CF  NREC = RECORD IDENTIFICATION, SHALL BE = 200 + NSATIN
CF  DAYBEG = BEGIN TIME OF THE RECORD (MJD)
CF  DAYEND = END TIME OF THE RECORD (MJD)
CF  EPOCH = EPOCH OF REFERENCE STATE VECTOR (MJD)
CF  REVEPO = REVOLUTION NUMBER AT EPOCH
CF  SMAXIS = SEMIMAJOR AXIS FOR THE KEPLER ORBIT
CF  OMOTIN = INVERSE MEAN MOTION FOR THE KEPLER ORBIT
      READ(LFILE,42,ERR=505,END=509)
     &NREC,DAYBEG,DAYEND,EPOCH,REVEPO,SMAXIS,OMOTIN

42    FORMAT(I3,2F12.6,F15.9,F11.3,2F13.5)
C  CHECK CONSISTENCY OF FILE
      IF(NREC .NE. 200 + NSATIN) GOTO 504
      IF(DAYBEG .GT. DAYEND) GOTO 504
C
C  DAYFIR = START TIME OF 1ST RECORD ON 1ST BLOCK ON THE FILE - MARGIN
      DAYFIR = DMIN1(DAYFIR,DAYBEG - 1.D-4)
C  ERROR RETURN IF 'DAY' IS BEFORE START OF FILE (WITH MARGIN)
      IF(DAY .LT. DAYFIR) THEN
        X(2) = DAYFIR
        GO TO 501
      ENDIF
C  ERROR RETURN IF THERE IS A GAP FROM LAST BLOCK (WITH MARGIN)
      IF(DAYBEG .GT. DAYLAS) GOTO 504                     ! PREVIOUSLY IERROR=3
C  DAYLAS = END TIME OF LAST READ RECORD BLOCK + MARGIN
      DAYLAS = DAYEND + 2.D-4
C
C  CONTINUE READ IF 'DAY' IS AFTER END OF THIS RECORD BLOCK
      IF(DAY .GT. DAYEND + 1.D-4) GOTO 20
C  REWIND WHEN 'DAY' IS EARLIER THAN START OF PRESENT RECORD BLOCK
      IF(DAY .LT. DAYBEG - 1.D-4) GOTO 10
C
C  READ 3RD RECORD IN THE BLOCK
CF  NREC = RECORD IDENTIFICATION, SHALL BE = 300 + NUMBER OF POL.COEFF.
CF  Y(6) = REFERENCE STATE VECTOR FOR KEPLER ORBIT (KM, KM/S)
CF  RDIST = S/C EARTH CENTRE DISTANCE AT EPOCH
      READ(LFILE,43,ERR=505,END=504) NREC,Y,RDIST
43    FORMAT(I3,3F11.3,3F11.7,F11.3)
 
C  CHECK CONSISTENCY OF FILE
      IF(NREC .GT. 310) GOTO 504
      IF(NREC .LT. 300) GOTO 504
C   KOEFF = NUMBER OF POLYNOMIAL COEFFICIENTS, BETWEEN 0 AND 10
      KOEFF = NREC - 300
C
C  IF THERE ARE NO COEFFICIENTS IN THIS BLOCK
      IF(KOEFF. LE. 0) GOTO 70
      DO 60 K = 1,KOEFF
CF  NREC = RECORD IDENT. = KOEFF + 11*K
CF  COEFF(10,6) = MATRIX WITH UP TO 10 COEFFICIENTS OF THE CHEBYSHEV
CF  POLYNOMIAL FOR EACH OF THE 6 COMPONENTS OF THE STATE VECTOR
      READ(LFILE,44,ERR=505,END=504) NREC,(COEFF(K,I),I=1,6)
44    FORMAT(I3,3F11.3,3F11.7)
C
C  CHECK CONSISTENCY OF FILE
      IF(11*K + KOEFF .NE. NREC) GOTO 504
60    CONTINUE
C  END OF BLOCK READING SEQUENCE, PROVIDE OUTPUT DATA
70    NSAT = NSATIN
C
C  TIME CONVERTED TO DIFFERENCE IN MEAN ANOMALY
      DMANOM = (DAY - EPOCH)*864.D2/OMOTIN
C  ORBIT NUMBER
      REVNUM = REVEPO + DMANOM/6.2831853072D0
C
C  START MODELLING KEPLER ORBIT
      ARIN = SMAXIS/RDIST
      ARM = (RDIST - SMAXIS)/SMAXIS
      RVWAM = (Y(1)*Y(4) + Y(2)*Y(5) + Y(3)*Y(6))*OMOTIN/SMAXIS**2
C  CALC. OF ECC. ANOMALY BY NEWTON'S ITERATION
      TAM = DMANOM - RVWAM
      COMP = 1.D-7 + 1.D-10*DABS(TAM)
      B = TAM
C  ITERATIONS TO SOLVE KEPLER'S EQUATION:
      DO 130 ITER = 1,15
      GO = DCOS(B)
      G1 = DSIN(B)
      BET = TAM - ARM*G1 + RVWAM*GO
      D = (BET - B)/(1.D0 + ARM*GO + RVWAM*G1)
      B = B + D
C  THIS GIVES THE ACCURACY  1.D-14 IN B & THE G'S
      IF(DABS(D) .LE. COMP) GOTO 140
130   CONTINUE
C  NO CONVERGENCE, ERROR RETURN
      GOTO 504
140   CONTINUE
      GO = GO - D*G1
      G1 = G1 + D*GO
      G2 = 1.D0 - GO
      G3 = B - G1
      FX = 1.D0  - G2*ARIN
      GX = (DMANOM - G3)*OMOTIN
C
      K = MIN0(KODE,3)
      DO 150 J = 1,K
150   X(J) = FX*Y(J) + GX*Y(J+3)
C
      IF(KODE .LE. 3) GOTO 170
      RX = DSQRT(X(1)**2 + X(2)**2 + X(3)**2)
      FT = -G1*SMAXIS*ARIN/(OMOTIN*RX)
      GT = 1.D0 - G2*SMAXIS/RX
      DO 160 J = 4,KODE
160   X(J) = FT*Y(J-3) + GT*Y(J)
C  END OF MODELLING KEPLER ORBIT
170   CONTINUE
C
C  CHECK IF POLYNOMIAL COEFFICIENTS ARE REQUIRED (1 IS NOT WORTH WHILE)
      IF(KOEFF .LE. 1) GOTO 600
C  MID-POINT & SCALE FACTOR FOR CHEBYSHEV POLYNOMIAL
      DAYMID = 0.5D0*(DAYBEG + DAYEND)
      SCALE = 4.D0/(DAYEND - DAYBEG)
C  ADD CHEBYSHEV POLYNOMIAL TO KEPLER STATE VECTOR
      S = SCALE*(DAY - DAYMID)
      PA = 1.D0
      P = S*0.5D0
C
C  'KODE' = NUMBER OF COMPONENTS OF THE STATE VECTOR
      DO 200 J = 1,KODE
200   X(J) = X(J) + COEFF(1,J) + COEFF(2,J)*P
C
      IF(KOEFF .LE. 2) GOTO 600
      DO 210 L = 3,KOEFF
      PB = PA
      PA = P
      P = S*PA - PB
      DO 210 J = 1,KODE
210   X(J) = X(J) + COEFF(L,J)*P
230   CONTINUE
C
      GOTO 600
C  ERROR RETURNS; IERROR = 7, 5, 4, 3, 2 OR 1
509   CONTINUE
C  END-OF-FILE ONLY IF AT LEAST ONE RECORD HAS BEEN READ
      IF(DAYLAS .LT. 1.D9) THEN
        X(3) = DAYLAS
        GOTO 502
      ENDIF
507   IERROR = IERROR + 2
505   IERROR = IERROR + 1
504   IERROR = IERROR + 1
503   IERROR = IERROR + 1
502   IERROR = IERROR + 1
501   IERROR = IERROR + 1
C  FORCE A RE-INITIALISATION OF READ AT NEXT CALL AFTER AN ERROR
      MFILE = -9999
600   RETURN
      END
