C ESA (C) 2000-2018 
C
C This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
C
C    SAS is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.
C
C    SAS is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.
C
C    You should have received a copy of the GNU General Public License
C    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
C
      SUBROUTINE SIGDEL(LFILE,MFILE,NSTA,DAY,DELAY,NSAT,IERROR)
C
C   SUBROUTINE FOR FINDING THE SIGNAL DELAY TIME FROM A 
C   SATELLITE TO A GROUND STATION. CALLS  ORBIT RETRIEVAL
C   SUBROUTINE "ORBITA".
C
CI  LFILE: UNIT NUMBER OF ORBIT FILE
CI  MFILE: UNIT NUMBER OF STATION FILE
CI  NSTA: STATION NUMBER
CI  DAY = MJD2000 OF EPOCH
C
CO  DELAY = SIGNAL TIME (SEC) FROM SATELLITE TO GROUND STATION
CO  NSAT = SATELLITE NUMBER
CO  IERROR = RETURN CODE FROM ORBIT FILE: 0=NO ERROR, 
C            1='DAY' TOO EARLY,
C            2=DAY TOO LATE, 
C            3=TIME GAP IN DATA,
C            4=NOT EXISTING STATION NR. IN STATION COORD. FILE 
C            5=ORBIT FILE CONTENT INCONSISTENT, 
C            6=READ ERROR FROM DATA FILE
C
      IMPLICIT NONE
C 
C
      INTEGER LFILE,MFILE,NSTA,NSAT,IERROR,IERR
      REAL*8  DAY,DELAY,S(4),Z(3),X(6),ZG,TH,GR,CGRZ,SGRZ,CZG,SZG
     #     ,CTH,STH,XGW,YGW,XGH,AN(7),TDA(3,3)
     #     ,REVNUM
      CHARACTER*8  ANAME
C
C
C  SIMPLIFIED GREENWHICH SIDEREAL ANGLE
C
      GR = 0.17447671633306130D+01 + 0.63003880989848886D+01*DAY
C
C  JSH - DON"T BORTH REMEMBERING, JUST GET THE STATION, DAMMIT
C  RETRIEVE SATION COORDINATES IF NSTAOLD DIFFERENT FROM NSTA
C
      CALL STATON(MFILE,NSTA,S,AN,TDA,ANAME,IERR)  
      IF (IERR.NE.0) THEN
         IF (IERR .EQ. 2801) IERROR = 4
         IF (IERR .EQ. 2802) IERROR = 6
         RETURN
      ENDIF
C
C   READ SPACECRAFT POSITION VECTOR X(3) IN INERTIAL MEAN-OF-2000 SYSTEM
C
      CALL ORBITA(DAY,6,LFILE,IERROR,NSAT,X,REVNUM)
C   CHECK READ ERROR
      IF(IERROR .EQ. 0) GOTO 1
      IF(IERROR.EQ.4 .OR. IERROR.EQ.5) IERROR=IERROR+1
      IF(IERROR.EQ.7)                  IERROR=6
      RETURN
C
C  SIMPLIFIED PRECESSION CONVERSION TO J2000.0
C
    1 ZG = (DAY - 0.5D0)*0.3061153D-6
      TH = (DAY - 0.5D0)*0.2660417D-6
C
      CGRZ = DCOS(GR - ZG)
      SGRZ = DSIN(GR - ZG)
      CZG=DCOS(ZG)
      SZG=DSIN(ZG)
      CTH=DCOS(TH)
      STH=DSIN(TH)
C
      XGW = S(1)*CGRZ - S(2)*SGRZ
      YGW = S(1)*SGRZ + S(2)*CGRZ
      XGH = CTH*XGW + STH*S(3)
C  Z(3) = STATION POSITION VECTOR IN INERTIAL MEAN-OF-2000 SYSTEM
      Z(1) =  CZG*XGH + SZG*YGW
      Z(2) = CZG*YGW - SZG*XGH 
      Z(3) = CTH*S(3) - STH*XGW
C
C  DISTANCE (KM) DIVIDED BY SPEED OF LIGHT (KM/S)
C
      DELAY = DSQRT((Z(1) - X(1))**2 +(Z(2) - X(2))**2 +
     & (Z(3) - X(3))**2)/299792.458 D0
C
C  SATELLITE POSITION AT TIME "DAY" - DELAY
C
      X(1) = X(1) - DELAY*X(4)
      X(2) = X(2) - DELAY*X(5)
      X(3) = X(3) - DELAY*X(6)
C
      DELAY =  DSQRT((Z(1) - X(1))**2 +(Z(2) - X(2))**2 +
     & (Z(3) - X(3))**2)/299792.458 D0
C
      RETURN
      END


