C ESA (C) 2000-2018 
C
C This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
C
C    SAS is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.
C
C    SAS is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.
C
C    You should have received a copy of the GNU General Public License
C    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
      SUBROUTINE STATON(MFILE,NSTA,R,AN,TDA,ANAME,IERR)                       
C
C     VERSION FROM FEBRUARY 3-RD , 1998
C     UPDATED BY FABIENNE DELHAISE FOR XMM PROJECT
C
C     READS STATION COORDINATES DATA FROM FILE:                         
C     ON UNIT "MFILE"                 
C     THE FILE IS A FORMATTED SEQUENTIAL FILE.                       
C
C  INPUT: 
C     MFILE    I*4:   LOGICAL UNIT NUMBER OF STATION COORD FILE     
C     NSTA     I*4    STATION NUMBER                                    
C                                                                       
C  OUTPUT:
C     R(4)     R*8    VECTOR CONTAINING GEOCENTRIC EARTH-ROTATING       
C                     CARTESIAN COORDINATES AND GEOCENTRIC RADIUS       
C                     VECTOR                                            
C     AN(7)    R*8    VECTOR CONTAINING STATION ALTITUDE ABOVE          
C                     GEOID, LATITUDE, ITS SINE AND COSINE,             
C                     LONGITUDE(EASTERN), ITS SINE AND COSINE           
C     TDA(3,3) R*8    MATRIX FOR TRANSFORMING GEOCENTRIC EARTH-         
C                     ROTATING COORDINATES(X) BY                        
C                     SUM TDA(J,K)*X(K), K=1,3  TO TOPOCENTRIC          
C                     COORDINATES: EAST, NORTH, ZENITH                  
C     ANAME  CHA*8    ALPHANUMERIC STATION NAME                         
C     IERR     I*4    RETURN CODE.    0 = NO ERROR.        
C                                  2801 = NOT EXISTING STATION NR. 
C                                  2802 = READ ERROR FROM STACO FILE.
      IMPLICIT REAL*8 (A-H,O-Z)
C                                                                       
C
      CHARACTER*1     CDUMMY                                         
      REAL*8          R(4),AN(7),TDA(3,3)                               
      CHARACTER*8     ANAME                                             
C
      DATA RAD/0.0174532925199433/
C                                                                       
C     REWIND, SKIP HEADER RECORD AND SEARCH FOR STATION NUMBER.
C
      REWIND MFILE
      READ(MFILE,'(A1)',END=90,ERR=99) CDUMMY
C
   10 READ(MFILE,11,END=90,ERR=99) ANAME,ISTA,XLON,XLAT,HEIGHT
   11 FORMAT(A8,I4,2F13.0,F10.0)
      IF (ISTA.NE.NSTA)        GOTO 10
C
C     STATION NUMBER FOUND
C                                 
      XLON=XLON*RAD                                                             
      XLAT=XLAT*RAD                                                             
      CALL GEOCOR(XLAT,XLON,HEIGHT,R,AN)
C
C     SET UP VALUES IN TDA FROM VALUES IN AN.                           
C
      TDA(1,1) = -AN(6)                                                 
      TDA(2,1) = -AN(3)*AN(7)                                           
      TDA(3,1) = AN(4)*AN(7)                                            
      TDA(1,2) = AN(7)                                                  
      TDA(2,2) = -AN(3)*AN(6)                                           
      TDA(3,2) = AN(4)*AN(6)                                            
      TDA(1,3) = 0.D0                                                   
      TDA(2,3) = AN(4)                                                  
      TDA(3,3) = AN(3)                                                  
C                                                                       
      IERR = 0                                                          
      RETURN                                                            
C                                                                       
C     ERROR                                                             
C                                                                       
   90 IERR = 2801                                                        
      RETURN                                                            
   99 IERR = 2802                                                        
      RETURN                                                            
      END  
C-----------------------------------------------------------------------
C                                                             
      SUBROUTINE GEOCOR (PHID,XLE,XH,R,AN)
C
C   UPDATED BY FABIENNE DELHAISE FOR XMM PROJECT
C   FEBRUARY 3-RD , 1998
C
CP  COMPUTES STATION COORDINATES FROM GEODETIC LATITUDE,
CP  LONGITUDE AND HEIGH
C
CI   PHID = GEODETIC LATITUDE
CI   XLE = EAST LONGITUDE OF THE STATION
CI   XH = ALTITUDE OF THE STATION ABOVE THE SURFACE OF THE
CI        REFERENCE ELLIPSOID
C
CO  R(1)..(3) = STATION COORDINATES IN THE ROTATING EARTH-SYSTEM WITH
CO              THE X-AXIS POINTING TOWARDS THE GREENWHICH MERIDIAN.
CO  R(4) = DSQRT(R(1)**2 + R(2)**2 + R(3)**2)
CO  AN(1)..(7) = VECTOR CONTAINING STATION ALTITUDE ABOVE GEOID,
CO               LATITUDE,ITS SINE & COSINE FOLLOWED BY LONGITUDE,
CO               ITS SINE & COSINE(EASTERN)
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION R(4),AN(7)
C
      DATA RE/6378.136/, FLATSQ/0.00669438499958795/
C
      SPHI =DSIN(PHID)
      AN(1) = XH
      AN(2) = PHID
      AN(3) = SPHI
      AN(4) = DCOS(PHID)
      AN(5) =XLE
      AN(6) = DSIN(XLE)
      AN(7) = DCOS(XLE)
      DEN = RE/DSQRT(1.D0  - SPHI*SPHI*FLATSQ)
      XC=   (DEN+XH)*AN(4)
      ZC = ((1.D0-FLATSQ)*DEN+XH)*SPHI
      R(1)=XC*AN(7)
      R(2)=XC*AN(6)
      R(3)=ZC
      R(4)=DSQRT(XC*XC+ZC*ZC)
      RETURN
      END
C-----------------------------------------------------------------------
